package com.qunzhixinxi.hnqz.admin.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.common.core.util.R;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报告导出--服务类
 * </p>
 *
 * @author hnqz
 * @since 2018-01-20
 */
public interface ReportExportService {

	public R getExportPackageZbTotalList(Page page, WmScorePackage wmScorePackage);

	public R getExportPackageZbTaskList(Page page, WmTask wmTask);

	public R exportNb(WmTask wmTask, WmReport wmReport) throws IOException;

	public R exportNba(WmTask wmTask,WmScorePackage wmScorePackage) throws IOException;

	public List<Map<String, Object>> exportNbInfo(WmTask wmTask) throws IOException;

	public R exportZb(WmTask wmTask) throws IOException;

	public List<Map<String, Object>> exportZbInfo(WmTask wmTask) throws IOException;

	R exportZbJs(WmTask wmTask) throws IOException;

	/**
	 * 导出患者教育
	 *
	 * @param packageId
	 * @return
	 */
	R exportHCP(String packageId) throws IOException;
}
